package conversion;

public class BaseXToBase10 {
	
	private int baseX;
	
	public BaseXToBase10(){
			baseX = 3;
	}
	
	public BaseXToBase10(int base){
		setBase(base);
	}
	
	/**
	 * Set the base of the class
	 * @param base the base of the class (IE 27 = Base27)
	 */
	public void setBase(int base){
		if(base > 1)
			baseX = base;
		else
			throw new IllegalArgumentException("You can not set a base below 2");
	}
	
	/**
	 * Turn base10 into base X (an array of base X)
	 * 		EX:	base27 the ArrayList of base 27
	 * 					base27[0] = value x 27^0
	 * 					base27[1] = value x 27^1
	 * 					base27[2] = value x 27^2
	 * 
	 * @param base10 an integer in base 10
	 * @return the array list representation of base X
	 */
	public int[] getBaseX(int base10){
		if(base10<0)
			return null;
		if(base10==0)
			return new int[]{0};
		
		int highestBase = 0;
		
		while(true){
			int temp = (int)(base10 / Math.pow( baseX, highestBase));
			if(temp >= baseX){
				highestBase++;
			}
			else break;
		}
		
		int[] baseXarray = new int[highestBase+1];
		
		for(int i=highestBase; i>=0; i--){
			baseXarray[i] = (int)(base10 / Math.pow(baseX,i));
			base10 = base10 % (int)Math.pow(baseX,i);
		}
		return baseXarray;
	}
	
	/**
	 * Turn baseX (which is in an integer array)
	 * 
	 * @param baseX the arraylist of base X
	 * 					base27[0] = value x 27^0
	 * 					base27[1] = value x 27^1
	 * 					base27[2] = value x 27^2
	 * @return the value of the base 27 in positive base 10, -1 is returned
	 * 			if the param base27 = null;
	 */
	public int getBase10(int[] baseXarray){
		
		//I should probably check the array to make sure it's the proper base
		
		if(baseXarray == null)
			return -1;
		
		int base10 = 0;
		for(int i=0; i<baseXarray.length;i++){
			base10 += (int)(baseXarray[i]* Math.pow(baseX,i));
		}
		
		return base10;
	}
}
