package conversion;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

public class ModifyImage {
	
	public static int[][] shrink(int[][] pixels){
        int width = pixels.length;
        int height = pixels[0].length;
        
        int[][] shrunkPixels = new int[width/2][height/2];
        
        for (int x = 0; x < width/2; x++) {
            for (int y = 0; y < height/2; y++) {
            	shrunkPixels[x][y] = 
            	averageFourPixels(pixels[x*2][y*2],  pixels[x*2+1][y*2],
            	                  pixels[x*2][y*2+1],pixels[x*2+1][y*2+1]);
            }
        }
        
        return shrunkPixels;
	}
	
    /*
     * averages each color (R,G,B) from four input pixels 
     */
    private static int averageFourPixels(int pixel1, int pixel2, int pixel3, int pixel4){
    	return makePixel(
    	(getRed(pixel1)   + getRed(pixel2)   + getRed(pixel3)   + getRed(pixel4)) /4,
    	(getGreen(pixel1) + getGreen(pixel2) + getGreen(pixel3) + getGreen(pixel4)) /4,
    	(getBlue(pixel1)  + getBlue(pixel2)  + getBlue(pixel3)  + getBlue(pixel4)) /4);	
    }
    
    public static int averagePixels(ArrayList<Integer> pixelsToAverage){
    	if(pixelsToAverage.size()==0){
    		System.err.println("You can not average 0 pixels.");
    		throw new IllegalArgumentException();
    	}
    	int totalPixels = pixelsToAverage.size();
    	int redTotal = 0;
    	int greenTotal = 0;
    	int blueTotal = 0;
    	
    	for(int i=0; i < totalPixels; i++){
    		int temp = pixelsToAverage.get(i);
    		redTotal += getRed(temp);
    		greenTotal += getGreen(temp);
    		blueTotal += getBlue(temp);
    	}
    	
    	return makePixel((redTotal/totalPixels),
    					 (greenTotal/totalPixels),
    					 (blueTotal/totalPixels));
    }
    
	/**
	 * Extracts the red component of a pixel.
	 *
	 * @param pixel	an integer pixel
	 *
	 * @return	the red component [0-255] of the pixel.
	 */
	public static int getRed(int pixel) {
		return pixel >> 16 & 0xff;
	}

	/**
	 * Extracts the green component of a pixel.
	 *
	 * @param pixel	an integer pixel
	 *
	 * @return	the green component [0-255] of the pixel.
	 */
	public static int getGreen(int pixel) {
		return pixel >> 8 & 0xff;
	}

	/**
	 * Extracts the blue component of a pixel.
	 *
	 * @param pixel	an integer pixel
	 *
	 * @return	the blue component [0-255] of the pixel.
	 */
	public static int getBlue(int pixel) {
		return pixel & 0xff;
	}
	
	/**
	 * Constructs a pixel from RGB components.
	 *
	 * @param red	the red component [0-255]
	 * @param green	the green component [0-255]
	 * @param blue	the blue component [0-255]
	 *
	 * @return	the packed integer pixel.
	 */
	public static int makePixel(int red, int green, int blue) {
		return (red & 0xff) << 16 | (green & 0xff) << 8 | (blue & 0xff);
	}
	
	/**
	 * Converts a two-dimensional array of image data to a BufferedImage.
	 *
	 * @param pixels	the source two-dimensional array of image data
	 *
	 * @return	a BufferedImage representing the source image.
	 *
	 * @throws	IllegalArgumentException if the source image is
	 *       	ill-defined.
	 */
	public static BufferedImage pixelsToImage(int[][] pixels) throws IllegalArgumentException {
		if (pixels == null) {
			throw new IllegalArgumentException();
		}

		int width = pixels[0].length;
		int height = pixels.length;
		BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		for (int row = 0; row < height; row++) {
			image.setRGB(0, row, width, 1, pixels[row], 0, width);
		}
		return image;
	}
	
	/**
	 * Converts a BufferedImage to a two-dimensional array of image data.
	 *
	 * @param image	the source BufferedImage
	 *
	 * @return	a two-dimensional array of image data representing the
	 *        	source image
	 *
	 * @throws	IllegalArgumentException if the source image is
	 *       	ill-defined.
	 */
	public static int[][] imageToPixels(BufferedImage image) throws IllegalArgumentException {
		if (image == null) {
			throw new IllegalArgumentException();
		}

		int width = image.getWidth();
		int height = image.getHeight();
		int[][] pixels = new int[height][width];
		for (int row = 0; row < height; row++) {
			image.getRGB(0, row, width, 1, pixels[row], 0, width);
		}
		return pixels;
	}
	
	public static void modifyPixel(int[][] image, int x, int y, int modifyValue){
		image[x][y]+=modifyValue;
	}
	
	public static char unmodifyPixel(int[][]orig, int[][] encrypted, int x, int y){
		return (char) (encrypted[x][y]-orig[x][y]);
	}
}
