/**
 * 
 */
package conversion;

import java.util.HashMap;

	/**
	 * @author Shane
	 * This class maps the X possiblities to a single value of R, G or B
	 * If the max difference is set to 1, then the possibilities are 1*2+1
	 * So 3, because +1, -1 and 0 are all possible values.
	 */
public class RGBDifferenceToInteger {
		/**
		 * This map links all of the RGB modifications to an integer.
		 */
	private HashMap<Integer, Integer> rgbDiffToInteger;
		/**
		 * This map links all of the integers to the RGB code.
		 */
	private HashMap<Integer, Integer> integerToRGBDiff;
	
	private int maxPixelDifference;
	private int base;
	
	/**
	 * Default constructor default maxPixelDiffernce is 3
	 */
	public RGBDifferenceToInteger(){
		this(3);
	}
	
	public RGBDifferenceToInteger(int maxPixelDifference){
			//check for larger later
		if(maxPixelDifference < 1)
			throw new IllegalArgumentException("The maxPixelDifference must be larger than 0");
		
			/**
			 * This maps all of the differences of the RGB values
			 * (IE: -1 to 0 and 0 to 1 and 1 to 2)
			 * integers.
			 */
		
		this.maxPixelDifference = maxPixelDifference;
		
			//this sets the base. maxPixelDifference = 1
			//then base = 3 because +1, -1 and 0
		base = maxPixelDifference*2 + 1;
		
		int integer = 0;
		
		rgbDiffToInteger = new HashMap<Integer, Integer>();
		for(int i = 0; i <= maxPixelDifference; i++){
			rgbDiffToInteger.put(i,integer);
			integer++;
		}
		
		for(int i = -(maxPixelDifference); i < 0; i++){
			rgbDiffToInteger.put(i,integer);
			integer++;
		}
		
		

		
			/**
			 * This maps all of the integers to
			 * all of the differences of a R, G, or B differences
			 * EX: if maxPixelDifference = 2 then
			 * -2 -> 0 and -1 -> 1 and 0 -> 2 and 1 -> 3 and 2 -> 4
			 */
		integerToRGBDiff = new HashMap<Integer, Integer>();
		
		integer = 0;
		
		integerToRGBDiff = new HashMap<Integer, Integer>();
		for(int i = 0; i <= maxPixelDifference; i++){
			integerToRGBDiff.put(integer,i);
			integer++;
		}
		for(int i = -(maxPixelDifference); i < 0; i++){
			integerToRGBDiff.put(integer,i);
			integer++;
		}
	}
	
	/**
	 * The difference of a single R, G or B of the RGB values is input as an integer
	 * and the integer equivalent is returned.  The differences can be 0, 1 or -1 if
	 * the maxPixelDifference is 1.
	 * @param difference a integer of the RGB differences (IE: -1, 0, 1)
	 * @return the integer equivalent to the input RGB differences
	 */
	public int getInteger(Integer difference){
		return rgbDiffToInteger.get(difference);
	}
	
	/**
	 * An integer is input and the RGB differences that that integer is mapped to
	 * is returned in a 3 element short array. The differences can be 0, 1 or -1.
	 * @param an integer (0 to base-1)
	 * @return an integer -maxDifference to +maxDifference
	 */
	public int getRGBDifferences(int integer){
		return integerToRGBDiff.get(integer);
	}
}