/**
 * 
 */
package tstego;

import java.awt.image.*;
import java.util.ArrayList;

import conversion.ModifyImage;

/**
 * @author Shane
 *
 */
public class CreateThumbnail {
	
	public static BufferedImage createThumbPercent(BufferedImage sourceImage, double percent){
		int width = (int)(sourceImage.getWidth() * percent);
		return createThumb(sourceImage, 'W', width);
	}
	
	//This won't work right because of double averaging by turning it into a width
	// when we turn it back into a height later on we'll get some off by one quite often
	public static BufferedImage createThumbHeight(BufferedImage sourceImage, int height){
		return createThumb(sourceImage, 'H', height);
	}
	
	public static BufferedImage createThumbWidth(BufferedImage sourceImage, int width){
		return createThumb(sourceImage, 'W', width);
	}
	
	/**
	 * creates the thumbnail of the image based on the width or height set
	 * @param sourceImage The image to be turned into the thumbnail
	 * @param type either W for width or H for height (may turn into boolean later)
	 * @param widthOrHeight is either the width or the height based on the param type.
	 * @return the thumbnail is returned
	 */
	private static BufferedImage createThumb(BufferedImage sourceImage, char type, int widthOrHeight){
		int[][] pixels = ModifyImage.imageToPixels(sourceImage);
		int[][] thumbPixels;
		
		int thumbWidth = 0, thumbHeight=0;
		
		if(type == 'W'){
			thumbWidth = widthOrHeight;
			thumbHeight = getThumbHeight(thumbWidth, pixels[0].length, pixels.length);
		}
		else if(type == 'H'){
			thumbHeight = widthOrHeight;
			thumbWidth = getThumbWidth(thumbHeight, pixels[0].length, pixels.length);
		}
		else{
			System.err.println("The thumbnail config had some sort of impossible disaster.");
			System.exit(1);
		}
		
		if (thumbWidth > sourceImage.getWidth() || thumbHeight > sourceImage.getHeight())
			throw new IllegalArgumentException("You can not create a thumbnail larger" +
					" than the source image.");
		
		thumbPixels = new int[thumbHeight][thumbWidth];

		double xPixelsToSkip = (double)pixels.length/(double)thumbHeight;
		double yPixelsToSkip = (double)pixels[0].length/(double)thumbWidth;
		
		/**
		 * This takes a single pixel and puts it
		 * into the new one. Method with different averaging could
		 * be added later (Evens, odds, every 3rd..., only first).
		 */
		for(int y=0; y<thumbWidth; y++){
			for(int x=0; x<thumbHeight; x++){
				/**
				 * puts all the pixels in the rectangular matrix until
				 * the next pixel to be stored and averages them.
				 */
				ArrayList<Integer> pixelsToAverage = new ArrayList<Integer>();
				for(int i=(int)(y*yPixelsToSkip); i<(int)(y*yPixelsToSkip+yPixelsToSkip)
															&& i<pixels[0].length; i++){
					for(int j=(int)(x*xPixelsToSkip); j<(int)(x*xPixelsToSkip+xPixelsToSkip)
															&& j<pixels.length; j++){
						pixelsToAverage.add(pixels[j][i]);
					}
				}
				
					//add the ability to check config
					//and average the pixels differently based on it
					//ie average only certain pixels, even pixels, odd pixels etc
				thumbPixels[x][y] = ModifyImage.averagePixels(pixelsToAverage);
			}
		}
		
		return ModifyImage.pixelsToImage(thumbPixels);
	}
	
	public static int getThumbHeight(int width, int origWidth, int origHeight){
		return (width*origHeight)/origWidth;
	}
	
	public static int getThumbWidth(int height, int origWidth, int origHeight){
		return (height*origWidth)/origHeight;
	}
}
