package tstego;


import java.awt.image.BufferedImage;

import javax.imageio.ImageIO;
import java.io.*;
import java.util.LinkedList;

/**
 * @author Shane
 *
 */
public class TStego {

	/**
	 * @param args
	 */
	public static void main(String[] args){
		boolean encrypt = false;
		
		if(args.length == 0){
			System.err.println("The first argument must be an E or D for Encrypt of Decrypt respectively.");
			System.exit(1);
		}
		
		if(args[0].equalsIgnoreCase("e")){
			encrypt = true;
		}
		else if(args[0].equalsIgnoreCase("d")){
			encrypt = false;
		}
		else{
			System.err.println("Invalid argument: The first argument must be either E or D.");
			System.exit(1);
		}
		
		
		/**
		 * This turns the arguments into variables for the encrypt method
		 */
		if(encrypt){
			if(args.length == 1){
				System.err.println("The second argument must be the source image.");
				System.exit(1);
			}
			
			BufferedImage img = null;
			try{
				img = ImageIO.read(new File(args[1]));
			} catch (Exception e) {
				System.err.println("There was an error with your picture file.");
				System.exit(1);
			}
			
			if(args.length == 2){
				System.err.println("The second argument must be output filename.");
				System.exit(1);
			}
			
			File output = null;
			try{
				output = new File(args[2]);
			}catch(Exception e){
				System.err.println("There was a problem with the output filename.");
			}
			
			/**
			 * This reads in the file to be encrypted in the image
			 */
			File fileToBeEncrypted;
			FileInputStream fis = null;
			try{
				fileToBeEncrypted = new File(args[3]);
				fis = new FileInputStream(fileToBeEncrypted);
			}catch(Exception e){
				System.err.println("There was a problem with the file being encrypted.");
			}
			
			
			LinkedList<Integer> data = new LinkedList<Integer>();
			
			/**
			 * This adds all the data from the file into a LinkedList
			 * byte by byte.
			 */
			try{
				int temp;
				while((temp = fis.read())!=-1)
					data.add(temp);
			}catch(IOException e){
				System.err.println("There was an error with your file to be encrypted in the image.");
			}
			

			/**
			 * Create the original thumbnail
			 */
			try{
					// S = Smallest Possible Thumbnail
					// W = Set Width
					// H = Set Height
					// P = Set Percentage
				if(args[4].equalsIgnoreCase("S")){
					int temp = ThumbStego.estimateSmallestThumbnailPossible(img,data.size());
					img = CreateThumbnail.createThumbWidth(img, temp);
				}
				else if(args[4].equalsIgnoreCase("W"))
					img = CreateThumbnail.createThumbWidth(img, Integer.parseInt(args[5]));
				else if(args[4].equalsIgnoreCase("H"))
					img = CreateThumbnail.createThumbHeight(img, Integer.parseInt(args[5]));
				else if(args[4].equalsIgnoreCase("P"))
					img = CreateThumbnail.createThumbPercent(img, Integer.parseInt(args[5]));
				else{
					System.err.println("W, H or P needs to be set as the 5th argument");
					System.exit(1);
				}
			} catch (Exception e) {
				System.err.println("There was an error with the percentage to decrease your picture by.");
				System.exit(1);
			}
			
			if(args.length==4){
				System.err.println("You need to percentage of size to create the thumbnail.");
			}
			

			img = ThumbStego.embedData(img, data);
			
			try{
				ImageIO.write(img, "png", output);
			}catch(IOException e){
				System.err.println("Error writing output file.");
			}
		}
		
		
		/**
		 * This turns the arguments into variables for the decrypt method
		 */
		else{
			if(args.length == 1){
				System.err.println("The second argument must be the source image.");
				System.exit(1);
			}
			
			BufferedImage img = null;
			try{
				img = ImageIO.read(new File(args[1]));
			} catch (Exception e) {
				System.err.println("There was an error with your picture file.");
				System.exit(1);
			}
			/*if (img.getType() != BufferedImage.TYPE_INT_RGB) {
				System.err.println("Wrong type of image.");
				System.exit(1);
			}*/
			
			if(args.length == 2){
				System.err.println("The second argument must be encrypted image's filename.");
				System.exit(1);
			}
			
			BufferedImage encryptedImg = null;
			try{
				encryptedImg = ImageIO.read(new File(args[2]));
			} catch (Exception e) {
				System.err.println("There was an error with your encrypted picture file.");
				System.exit(1);
			}
			
			
			/**
			 * This creates the file to be decrypted from the image
			 */
			File fileToBeDecrypted;
			FileOutputStream fout = null;
			try{
				fileToBeDecrypted = new File(args[3]);
				fout = new FileOutputStream(fileToBeDecrypted);
			}catch(Exception e){
				System.err.println("There was a problem with the file being encrypted.");
			}
			
			
			
			
			/**
			 * Create the original thumbnail
			 */
			img = CreateThumbnail.createThumbWidth(img, encryptedImg.getWidth());
			
			
			/**
			 * extracts the data from the image
			 */
			LinkedList<Integer> data = ThumbStego.extractData(img, encryptedImg);
			
			/**
			 * This adds all the data from to file from a LinkedList
			 * byte by byte.
			 */
			try{
				for(int i=0; i<data.size();i++)
					fout.write(data.get(i));
			}catch(IOException e){
				System.err.println("There was an error with your file to be encrypted in the image.");
			}
			
			
		}
		
		System.out.println("Done!");

	}

}
