/**
 * 
 */
package ui;

import java.awt.event.*;

import java.awt.image.BufferedImage;

import java.io.*;

import javax.imageio.ImageIO;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.swing.*;
import javax.swing.filechooser.*;

import tstego.*;

/**
 * @author Shane Pope
 * This is a modification of Samir Vaidya's (mailto:syvaidya@gmail.com)
 * OpenStego UI (https://sourceforge.net/projects/openstego)
 */
public class ThumbStegoUI extends ThumbStegoFrame {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch(Exception e)
        {
        }
        new ThumbStegoUI().setVisible(true);
	}
	


    /**
     * Static variable to holds path to last selected folder
     */
    private static String lastFolder = ".";

	public ThumbStegoUI(){
		super();
		
		Listener listener = new Listener();
		addWindowListener(listener);

		srcImgFileButton.addActionListener(listener);
		dataFileButton.addActionListener(listener);
		srcImg2FileButton.addActionListener(listener);
		embThumbFileButton.addActionListener(listener);
		outputFileButton.addActionListener(listener);
		embedButton.addActionListener(listener);
		extractButton.addActionListener(listener);
	}
    
	/**
	 * This silly class takes the source file name
	 * and creates the thumbnail file named "sourceNameFile"_thumb.png
	 * @param sourceFileName the source image filename
	 * @return the thumbnail filename is returned.
	 */
	private String createThumbName(String sourceFileName){
		String tempArray[] = sourceFileName.split("\\.");
		StringBuffer thumbnailName = new StringBuffer();
		for(int i = 0; i < tempArray.length-1; i++){
			thumbnailName.append(tempArray[i]);
		}
		thumbnailName.append("_thumb.png");
		
		return thumbnailName.toString();
	}
	
    /**
     * Method to check whether value is provided or not; and display message box in case it is not provided
     * @param textField Text field to be checked for value
     * @param fieldName Name of the field
     * @return Flag whether value exists or not
     */
    private boolean checkMandatory(JTextField textField, String fieldName)
    {
        String value = textField.getText();

        if(value == null || value.trim().equals(""))
        {
            JOptionPane.showMessageDialog(this, new Object[] { fieldName },
            								"Missing file location",
            								JOptionPane.ERROR_MESSAGE);
            textField.requestFocus();
            return false;
        }

        return true;
    }
	
	/**
     * This method embeds the selected data file into selected image file
     * @throws OpenStegoException
     */
    private void embedData()
    {
        BufferedImage image = null;
        String dataFileName = null;
        String imgFileName = null;
        String outputFileName = null;
        File outputFile = null;

        dataFileName = dataTextField.getText();
        imgFileName = srcImgTextField.getText();
        outputFileName = createThumbName(imgFileName);

        // START: Input Validations
        if(!checkMandatory(srcImgTextField,  "The source image location was left blank.")) return;
        if(!checkMandatory(dataTextField, "The data file location was left blank.")) return;
        // END: Input Validations
        
        
        
        
        
        
        //Reading in Data
        /**
		 * This reads in the file to be encrypted in the image
		 */
		File fileToBeEmbeded;
		FileInputStream fis = null;
		try{
			fileToBeEmbeded = new File(dataFileName);
			fis = new FileInputStream(fileToBeEmbeded);
		}catch(Exception e){
			JOptionPane.showMessageDialog(this, "There was a problem with the file being encrypted.");
		}
		
		LinkedList<Integer> data = new LinkedList<Integer>();
		
		/**
		 * This adds all the data from the file into a LinkedList
		 * byte by byte.
		 */
		try{
			int temp;
			while((temp = fis.read())!=-1)
				data.add(temp);
		}catch(IOException e){
			JOptionPane.showMessageDialog(this, "There was an error with your file to be encrypted in the image.");
		}
		//Source DATA reading in ends
        
		
		
		
		
		
        //Thumbnail Resizing
        String buttonSelection = radioGroup.getSelection().getActionCommand();
        
        BufferedImage img = null;
		try{
			img = ImageIO.read(new File(imgFileName));
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "There was an error reading in the source image.");
			return;
		}
        
        if(buttonSelection.equals("WIDTH")){
        	if(!checkMandatory(widthTextField, "The width is not filled in.")) return;
        	int width = 0;
        	try{
        		width = Integer.parseInt(widthTextField.getText());
        	}
        	catch(NumberFormatException e){
        		JOptionPane.showMessageDialog(this, "You must instert a number in the width pixel text box.");
        		return;
        	}
        	if(width > img.getWidth()){
        		JOptionPane.showMessageDialog(this, "The width must be less than or equal to the source image width.");
        		return;
        	}
        	
        	img = CreateThumbnail.createThumbWidth(img, width);
        	
        }
        else if(buttonSelection.equals("HEIGHT")){
        	if(!checkMandatory(heightTextField, "The height is not filled in.")) return;
        	int height = 0;
        	try{
        		height = Integer.parseInt(heightTextField.getText());
        	}
        	catch(NumberFormatException e){
        		JOptionPane.showMessageDialog(this, "You must instert a number in the height pixel text box.");
        		return;
        	}
        	if(height > img.getHeight()){
        		JOptionPane.showMessageDialog(this, "The height must be less than or equal to the source image height.");
        		return;
        	}
        	img = CreateThumbnail.createThumbHeight(img, height);
        	
        }
        else if(buttonSelection.equals("PERCENTAGE")){
        	if(!checkMandatory(percentageTextField, "The percentage is not filled in.")) return;
        	double percentage = 0;
        	try{
        		percentage = Double.parseDouble(percentageTextField.getText());
        	}
        	catch(NumberFormatException e){
        		JOptionPane.showMessageDialog(this, "You must instert a number in the percentage pixel text box.");
        		return;
        	}
        	if(percentage<=0 || percentage > 1){
        		JOptionPane.showMessageDialog(this, "The percentage must be between 0.0 and 1.0");
        		return;
        	}
        		
        	img = CreateThumbnail.createThumbPercent(img, percentage);
        	
        }
        else if(buttonSelection.equals("SMALLEST")){
        	//Smallest
        	int temp = ThumbStego.estimateSmallestThumbnailPossible(img,data.size());
			img = CreateThumbnail.createThumbWidth(img, temp);
        }
        else{
        	JOptionPane.showMessageDialog(this, "Please select a method of resizing.");
        	return;
        }
        
        
        if(!ThumbStego.checkSize(img, data, 3)){
        	JOptionPane.showMessageDialog(this, "The data will not fit in the image selected at that size of a thumbnail.");
        }
        
        image = ThumbStego.embedData( img, data);

        
        outputFile = new File(outputFileName);

        if(outputFile.exists())
        {
            if(JOptionPane.showConfirmDialog(this, new Object[] { "Do you want to overwrite "+outputFileName },
            								 "Warning, File already exists",
            								 JOptionPane.YES_NO_OPTION,
            								 JOptionPane.WARNING_MESSAGE)
            								 == JOptionPane.NO_OPTION)
            {
                return;
            }
        }

        try
        {
            ImageIO.write(image, "png", outputFile);
        }
        catch(IOException ex)
        {
            JOptionPane.showMessageDialog(this, "There was an error with the image you were attempting to write to.");
        }
        JOptionPane.showMessageDialog(this, "The data has been Embedded successfully.");

        srcImgTextField.setText("");
        dataTextField.setText("");
        widthTextField.setText("");
        heightTextField.setText("");
        percentageTextField.setText("");
        
        srcImgTextField.requestFocus();
    }

    /**
     * This method extracts data from the selected image file
     * @throws OpenStegoException
     */
    private void extractData()
    {
        String sourceImgFileName = null;
        String embededThumbFileName = null;
        String outputFileName = null;
        File file = null;
        FileOutputStream fos = null;
        List<Integer> thumbOutput = null;

        sourceImgFileName = srcImg2TextField.getText();
        embededThumbFileName = embThumbTextField.getText();
        outputFileName = outputFileTextField.getText();

        // Input Validations
        if(!checkMandatory(srcImg2TextField, "Please select the source image file location")) return;
        if(!checkMandatory(embThumbTextField, "Please select the embed thumbnail image file location")) return;
        if(!checkMandatory(outputFileTextField, "Please select the output file location")) return;

        
        
        
        
        //READ IN THE IMAGES
        BufferedImage img = null;
		try{
			img = ImageIO.read(new File(sourceImgFileName));
		} catch (Exception e) {
			System.err.println("There was an error with your source image file.");
		}

		BufferedImage imgE = null;
		try{
			imgE = ImageIO.read(new File(embededThumbFileName));
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "There was an error with your embeded thumbnail picture file.");
		}
        
		
			//Create the original thumbnail
		img = CreateThumbnail.createThumbWidth(img, imgE.getWidth());
        
        try
        {
        	thumbOutput = ThumbStego.extractData(img, imgE);
        }
        catch(Exception e)
        {
        	System.err.println(e);
        	//Don't do nothing! Error is output from extractData method... bad
        	//design, fix later.
        }

        try
        {
            file = new File(outputFileName);
            if(file.exists())
            {
                if(JOptionPane.showConfirmDialog(this,
                		                        new Object[] { "File " + outputFileName + " already exists. Overwrite?" },
                								"File already Exists",
                								JOptionPane.YES_NO_OPTION,
                								JOptionPane.WARNING_MESSAGE)
                								== JOptionPane.NO_OPTION)
                {
                    return;
                }
            }
            fos = new FileOutputStream(file);
			for(int i=0; i<thumbOutput.size();i++)
				fos.write(thumbOutput.get(i));
            fos.close();
        }
        catch(IOException ioEx)
        {
            JOptionPane.showMessageDialog(this, "Error writing data file.");
        }

        JOptionPane.showMessageDialog(this, "Successfully extracted data.");

        srcImg2TextField.setText("");
        embThumbTextField.setText("");
        outputFileTextField.setText("");
        srcImg2TextField.requestFocus();
    }

    /**
     * This method shows the file chooser and updates the text field based on the selection
     * @author Samir Vaidya Modified by Shane Pope
     */
    private void selectFile(String action)
    {
        String fileName = null;
        String title = null;
        String filterDesc = null;
        JTextField textField = null;

        if(action.equals("BROWSE_DATA"))
        {
            title = "Select the source Data File...";
            textField = this.dataTextField;
        }
        else if(action.equals("BROWSE_SRC_IMG"))
        {
            title = "Select source image...";
            textField = this.srcImgTextField;
        }
        else if(action.equals("BROWSE_SRC2_IMG"))
        {
            title = "Select source image...";
            textField = this.srcImg2TextField;
        }
        else if(action.equals("BROWSE_EMBEDED_THUMB"))
        {
            title = "Select thumbnail with embeded data...";
            filterDesc = "png";
            textField = this.embThumbTextField;
        }
        else if(action.equals("BROWSE_OUTPUT_FILE"))
        {
            title = "Create the output file...";
            textField = this.outputFileTextField;
        }
        
        fileName = getFileName(title, filterDesc);
        
        if(fileName != null)
        {
            textField.setText(fileName);
        }
    }
    
    /**
     * Get's the filename using JFileChooser
     * @return
     */
    protected String getFileName(String title, String filterDesc){
	    int retVal = 0;
	    String fileName = null;
	    JFileChooser chooser = new JFileChooser(lastFolder);
	    
	    chooser.setDialogTitle(title);
	    

	    if( filterDesc != null ){
	    	FileNameExtensionFilter  fileFilter = new FileNameExtensionFilter(filterDesc + " type",filterDesc);
	    	chooser.setFileFilter(fileFilter);
	    }
	    
	    retVal = chooser.showOpenDialog(null);
	    
	    fileName = chooser.getSelectedFile().getPath();
	    lastFolder = chooser.getSelectedFile().getParent();
	    
	    if(retVal == JFileChooser.APPROVE_OPTION)
	    {
	        fileName = chooser.getSelectedFile().getPath();
	        lastFolder = chooser.getSelectedFile().getParent();
	    }
	    return fileName;
	}

    /**
     * Close the application
     * @author Samir Vaidya Modified by Shane Pope
     */
    private void close()
    {
        System.exit(0);
    }
    
    /**
     * Common listener class to handle action and window events
     * @author Samir Vaidya modified by Shane Pope
     */
    class Listener implements ActionListener, WindowListener
    {
        public void actionPerformed(ActionEvent ev)
        {
            try
            {
                String action = ev.getActionCommand();

                if(action.startsWith("BROWSE_"))
                {
                    selectFile(action);
                }
                else if(action.equals("EMBED"))
                {
                	embedData();
                }
                else if(action.equals("EXTRACT"))
                {
                	extractData();
                }
                else{
                	//do nothing :) (IE if they close the box)
                }
            }
            catch(Exception e)
            {
                //System.err.println(e);
            }
        }

        public void windowClosing(WindowEvent ev)
        {
            close();
        }

        public void windowActivated(WindowEvent ev) {}
        public void windowClosed(WindowEvent ev) {}
        public void windowDeactivated(WindowEvent ev) {}
        public void windowDeiconified(WindowEvent ev) {}
        public void windowIconified(WindowEvent ev) {}
        public void windowOpened(WindowEvent ev) {}
    }
    
}







